/* License. */
/*  */
/* geo_code.h */
/*  */
/* This file provides a set of geometric facilities. */



#ifndef GEO_CODE_H

# define GEO_CODE_H



# if !defined( USE_FLT ) && !defined( USE_DBL ) && !defined( USE_LDBL )
#  error "geo_code.h requires the macro USE_FLT, USE_DBL, or USE_LDBL to be defined."
# else



	/* Sets NAMESPACE to suit the macro-indicated float type. */
#  include "std_commonflt.h"



#  define VEC_PAR CONCAT( NAMESPACE, vectors_parallel )
#  define VEC_PERP CONCAT( NAMESPACE, vectors_perpendicular )



#  define TRI CONCAT( NAMESPACE, triangle )

#  define INIT_TRI CONCAT( NAMESPACE, initialize_triangle )
#  define PRINT_TRI CONCAT( NAMESPACE, print_triangle )
#  define TRI_AREA CONCAT( NAMESPACE, triangle_area )
#  define TRI_AREA_INLINE CONCAT( NAMESPACE, triangle_area_inline )
#  define TRI_NULL CONCAT( NAMESPACE, triangle_isnull )
#  define PT_IN_TRI CONCAT( NAMESPACE, point_in_triangle )
#  define TRI_IN_TRI CONCAT( NAMESPACE, triangle_in_triangle )

#  define FIND_BARY CONCAT( NAMESPACE, find_barycentric )
#  define USE_BARY CONCAT( NAMESPACE, use_barycentric )
#  define FIND_INTERS CONCAT( NAMESPACE, find_intersections )



#  define VS CONCAT( NAMESPACE, vector_set )

#  define INIT_VS CONCAT( NAMESPACE, init_vectorset )



/* Note: Add "triangle overlap" function to detect whether two triangles share points in common. */



int VEC_PAR ( FLT_TYPE x1, FLT_TYPE y1,  FLT_TYPE x2, FLT_TYPE y2 );
int VEC_PERP ( FLT_TYPE x1, FLT_TYPE y1,  FLT_TYPE x2, FLT_TYPE y2 );



typedef struct TRI
{
	FLT_TYPE x[ 3 ], y[ 3 ];
	
} TRI;


int INIT_TRI ( TRI *t,  FLT_TYPE x1, FLT_TYPE y1,  FLT_TYPE x2, FLT_TYPE y2,  FLT_TYPE x3, FLT_TYPE y3 );

int PRINT_TRI( const char *preq, TRI *t, int sep_vert );

	/* Uses the cross-product method. */
int TRI_AREA ( TRI *t, FLT_TYPE *ret );
FLT_TYPE TRI_AREA_INLINE( TRI *t, int *ret );
	/* Use the barycentric system. */
int PT_IN_TRI ( TRI *area,  FLT_TYPE x, FLT_TYPE y );
int TRI_IN_TRI( TRI *outer, TRI *inner );
	/* Fairly simple. */
int TRI_NULL( TRI *t );

int FIND_BARY ( TRI *area,  FLT_TYPE x, FLT_TYPE y,    FLT_TYPE *bary1, FLT_TYPE *bary2 );
int USE_BARY ( TRI *area,  FLT_TYPE bary1, FLT_TYPE bary2,    FLT_TYPE *x, FLT_TYPE *y );

	/* retf[ i ]: */
	/*     0: t->[ 0 ] & t->[ 1 ] */
	/*     1: t->[ 1 ] & t->[ 2 ] */
	/*     2: t->[ 0 ] & t->[ 2 ] */
int FIND_INTERS ( TRI *t,  FLT_TYPE x1, FLT_TYPE y1,  FLT_TYPE x2, FLT_TYPE y2,    int *reti, FLT_TYPE *retf );



typedef struct VS
{
	size_t count;
	FLT_TYPE *x1, *y1,  *x2, *y2;
	
} VS;


int INIT_VS( TRI *src,  VS *vs, TRI *partner );

# endif

#endif
/* End geo_code.h */
